@@ -12,11 +12,11 @@ from ast import Module, Num, Expr, Str, Bytes, UnaryOp, UAdd, USub, Invert, \
     YieldFrom, Return, IfExp, Lambda, arguments, arg, Call, keyword, \
     Attribute, Global, Nonlocal, If, While, For, withitem, With, Try, \
     ExceptHandler, FunctionDef, ClassDef, Starred, NodeTransformer, \
-    Interactive, Expression, Index, literal_eval, dump, walk
-from ast import Ellipsis  # pylint: disable=redefined-builtin
+    Interactive, Expression, Index, literal_eval, dump, walk, increment_lineno
+from ast import Ellipsis as EllipsisNode
 # pylint: enable=unused-import
 import textwrap
-from itertools import repeat
+import itertools
 
 from xonsh.tools import subproc_toks, find_next_break
 from xonsh.platform import PYTHON_VERSION_INFO
@@ -60,6 +60,22 @@ def leftmostname(node):
     return rtn
 
 
+def get_lineno(node, default=0):
+    """Gets the lineno of a node or returns the default."""
+    return getattr(node, 'lineno', default)
+
+
+def min_line(node):
+    """Computes the minimum lineno."""
+    node_line = get_lineno(node)
+    return min(map(get_lineno, walk(node), itertools.repeat(node_line)))
+
+
+def max_line(node):
+    """Computes the maximum lineno."""
+    return max(map(get_lineno, walk(node)))
+
+
 def get_col(node, default=-1):
     """Gets the col_offset of a node, or returns the default"""
     return getattr(node, 'col_offset', default)
@@ -67,7 +83,7 @@ def get_col(node, default=-1):
 
 def min_col(node):
     """Computes the minimum col_offset."""
-    return min(map(get_col, walk(node), repeat(node.col_offset)))
+    return min(map(get_col, walk(node), itertools.repeat(node.col_offset)))
 
 
 def max_col(node):
@@ -78,6 +94,30 @@ def max_col(node):
     return col
 
 
+def get_id(node, default=None):
+    """Gets the id attribute of a node, or returns a default."""
+    return getattr(node, 'id', default)
+
+
+def gather_names(node):
+    """Returns the set of all names present in the node's tree."""
+    rtn = set(map(get_id, walk(node)))
+    rtn.discard(None)
+    return rtn
+
+
+def has_elts(x):
+    """Tests if x is an AST node with elements."""
+    return isinstance(x, AST) and hasattr(x, 'elts')
+
+
+def xonsh_call(name, args, lineno=None, col=None):
+    """Creates the AST node for calling a function of a given name."""
+    return Call(func=Name(id=name, ctx=Load(), lineno=lineno, col_offset=col),
+                args=args, keywords=[], starargs=None, kwargs=None,
+                lineno=lineno, col_offset=col)
+
+
 def isdescendable(node):
     """Deteremines whether or not a node is worth visiting. Currently only
     UnaryOp and BoolOp nodes are visited.
@@ -104,6 +144,7 @@ class CtxAwareTransformer(NodeTransformer):
         self.contexts = []
         self.lines = None
         self.mode = None
+        self._nwith = 0
 
     def ctxvisit(self, node, inp, ctx, mode='exec'):
         """Transforms the node in a context-dependent way.
@@ -125,8 +166,10 @@ class CtxAwareTransformer(NodeTransformer):
         self.lines = inp.splitlines()
         self.contexts = [ctx, set()]
         self.mode = mode
+        self._nwith = 0
         node = self.visit(node)
         del self.lines, self.contexts, self.mode
+        self._nwith = 0
         return node
 
     def ctxupdate(self, iterable):
@@ -171,7 +214,7 @@ class CtxAwareTransformer(NodeTransformer):
             if not isinstance(newnode, AST):
                 # take the first (and only) Expr
                 newnode = newnode[0]
-            newnode.lineno = node.lineno
+            increment_lineno(newnode, n=node.lineno - 1)
             newnode.col_offset = node.col_offset
         except SyntaxError:
             newnode = node
@@ -191,6 +234,85 @@ class CtxAwareTransformer(NodeTransformer):
                 break
         return inscope
 
+    #
+    # With Transformers
+    #
+    def insert_with_block_check(self, node):
+        """Modifies a with statement node in-place to add an initial check
+        for whether or not the block should be executed. If the block is
+        not executed it will raise a XonshBlockError containing the required
+        information.
+        """
+        nwith = self._nwith  # the nesting level of the current with-statement
+        lineno = get_lineno(node)
+        col = get_col(node, 0)
+        # Add or discover target names
+        targets = set()
+        i = 0  # index of unassigned items
+
+        def make_next_target():
+            nonlocal i
+            targ = '__xonsh_with_target_{}_{}__'.format(nwith, i)
+            n = Name(id=targ, ctx=Store(), lineno=lineno, col_offset=col)
+            targets.add(targ)
+            i += 1
+            return n
+        for item in node.items:
+            if item.optional_vars is None:
+                if has_elts(item.context_expr):
+                    targs = [make_next_target() for _ in item.context_expr.elts]
+                    optvars = Tuple(elts=targs, ctx=Store(), lineno=lineno,
+                                    col_offset=col)
+                else:
+                    optvars = make_next_target()
+                item.optional_vars = optvars
+            else:
+                targets.update(gather_names(item.optional_vars))
+        # Ok, now that targets have been found / created, make the actual check
+        # to see if we are in a non-executing block. This is equivalent to
+        # writing the following condition:
+        #
+        #     if getattr(targ0, '__xonsh_block__', False) or \
+        #        getattr(targ1, '__xonsh_block__', False) or ...:
+        #         raise XonshBlockError(lines, globals(), locals())
+        tests = [_getblockattr(t, lineno, col) for t in sorted(targets)]
+        if len(tests) == 1:
+            test = tests[0]
+        else:
+            test = BoolOp(op=Or(), values=tests, lineno=lineno, col_offset=col)
+        ldx, udx = self._find_with_block_line_idx(node)
+        lines = [Str(s=s, lineno=lineno, col_offset=col)
+                 for s in self.lines[ldx:udx]]
+        check = If(test=test, body=[
+            Raise(exc=xonsh_call('XonshBlockError',
+                                 args=[List(elts=lines, ctx=Load(),
+                                            lineno=lineno, col_offset=col),
+                                       xonsh_call('globals', args=[],
+                                                  lineno=lineno, col=col),
+                                       xonsh_call('locals', args=[],
+                                                  lineno=lineno, col=col)],
+                                 lineno=lineno, col=col),
+                  cause=None, lineno=lineno, col_offset=col)],
+                orelse=[], lineno=lineno, col_offset=col)
+        node.body.insert(0, check)
+
+    def _find_with_block_line_idx(self, node):
+        ldx = min_line(node.body[0]) - 1
+        udx = max_line(node.body[-1])
+        # now check if parsable, or add lines until it is or we run out of lines
+        nlines = len(self.lines)
+        lines = 'with __xonsh_dummy__:\n' + '\n'.join(self.lines[ldx:udx])
+        lines += '\n'
+        parsable = False
+        while not parsable and udx < nlines:
+            try:
+                self.parser.parse(lines, mode=self.mode)
+                parsable = True
+            except SyntaxError:
+                lines += self.lines[udx] + '\n'
+                udx += 1
+        return ldx, udx
+
     #
     # Replacement visitors
     #
@@ -286,8 +408,11 @@ class CtxAwareTransformer(NodeTransformer):
         """Handle visiting a with statement."""
         for item in node.items:
             if item.optional_vars is not None:
-                self.ctxadd(leftmostname(item.optional_vars))
+                self.ctxupdate(gather_names(item.optional_vars))
+        self._nwith += 1
         self.generic_visit(node)
+        self._nwith -= 1
+        self.insert_with_block_check(node)
         return node
 
     def visit_For(self, node):
@@ -365,4 +490,19 @@ def pdump(s, **kwargs):
     return pre + mid + post
 
 
+def pprint_ast(s, *, sep=None, end=None, file=None, flush=False, **kwargs):
+    """Performs a pretty print of the AST nodes."""
+    print(pdump(s, **kwargs), sep=sep, end=end, file=file, flush=flush)
 
+
+#
+# Private helpers
+#
+
+def _getblockattr(name, lineno, col):
+    """calls getattr(name, '__xonsh_block__', False)."""
+    return xonsh_call('getattr', args=[
+        Name(id=name, ctx=Load(), lineno=lineno, col_offset=col),
+        Str(s='__xonsh_block__', lineno=lineno, col_offset=col),
+        NameConstant(value=False, lineno=lineno, col_offset=col)],
+        lineno=lineno, col=col)
