@@ -6,7 +6,7 @@ import builtins
 from xonsh.platform import ON_WINDOWS
 from xonsh.tools import (subexpr_from_unbalanced, get_sep,
                          check_for_partial_string, RE_STRING_START,
-                         iglobpath)
+                         iglobpath, levenshtein)
 
 from xonsh.completers.tools import get_filter_function
 
@@ -80,28 +80,36 @@ def _startswithnorm(x, start, startlow=None):
     return x.startswith(start)
 
 
-def _add_env(paths, prefix):
+def _env(prefix):
     if prefix.startswith('$'):
         key = prefix[1:]
-        paths.update({'$' + k
-                      for k in builtins.__xonsh_env__
-                      if get_filter_function()(k, key)})
+        return {'$' + k
+                for k in builtins.__xonsh_env__
+                if get_filter_function()(k, key)}
+    return ()
 
 
-def _add_dots(paths, prefix):
+def _dots(prefix):
+    slash = get_sep()
+    if slash == '\\':
+        slash = ''
     if prefix in {'', '.'}:
-        paths.update({'./', '../'})
-    if prefix == '..':
-        paths.add('../')
+        return ('.'+slash, '..'+slash)
+    elif prefix == '..':
+        return ('..'+slash,)
+    else:
+        return ()
 
 
 def _add_cdpaths(paths, prefix):
     """Completes current prefix using CDPATH"""
     env = builtins.__xonsh_env__
     csc = env.get('CASE_SENSITIVE_COMPLETIONS')
+    glob_sorted = env.get('GLOB_SORTED')
     for cdp in env.get('CDPATH'):
         test_glob = os.path.join(cdp, prefix) + '*'
-        for s in iglobpath(test_glob, ignore_case=(not csc)):
+        for s in iglobpath(test_glob, ignore_case=(not csc),
+                           sort_result=glob_sorted):
             if os.path.isdir(s):
                 paths.add(os.path.basename(s))
 
@@ -151,7 +159,82 @@ def _quote_paths(paths, start, end):
     return out
 
 
-def complete_path(prefix, line, start, end, ctx, cdpath=True):
+def _joinpath(path):
+    # convert our tuple representation back into a string representing a path
+    if path is None:
+        return ''
+    elif len(path) == 0:
+        return ''
+    elif path == ('',):
+        return get_sep()
+    elif path[0] == '':
+        return get_sep() + _normpath(os.path.join(*path))
+    else:
+        return _normpath(os.path.join(*path))
+
+
+def _splitpath(path):
+    # convert a path into an intermediate tuple representation
+    # if this tuple starts with '', it means that the path was an absolute path
+    path = _normpath(path)
+    if path.startswith(get_sep()):
+        pre = ('', )
+    else:
+        pre = ()
+    return pre + _splitpath_helper(path, ())
+
+
+def _splitpath_helper(path, sofar=()):
+    folder, path = os.path.split(path)
+    if path == "":
+        return sofar[::-1]
+    elif folder == "":
+        return (sofar + (path, ))[::-1]
+    else:
+        return _splitpath_helper(folder, sofar + (path, ))
+
+
+def subsequence_match(ref, typed, csc):
+    """
+    Detects whether typed is a subsequence of ref.
+
+    Returns ``True`` if the characters in ``typed`` appear (in order) in
+    ``ref``, regardless of exactly where in ``ref`` they occur.  If ``csc`` is
+    ``False``, ignore the case of ``ref`` and ``typed``.
+
+    Used in "subsequence" path completion (e.g., ``~/u/ro`` expands to
+    ``~/lou/carcohl``)
+    """
+    if csc:
+        return _subsequence_match_iter(ref, typed)
+    else:
+        return _subsequence_match_iter(ref.lower(), typed.lower())
+
+
+def _subsequence_match_iter(ref, typed):
+    if len(typed) == 0:
+        return True
+    elif len(ref) == 0:
+        return False
+    elif ref[0] == typed[0]:
+        return _subsequence_match_iter(ref[1:], typed[1:])
+    else:
+        return _subsequence_match_iter(ref[1:], typed)
+
+
+def _expand_one(sofar, nextone, csc):
+    out = set()
+    glob_sorted = builtins.__xonsh_env__.get('GLOB_SORTED')
+    for i in sofar:
+        _glob = os.path.join(_joinpath(i), '*') if i is not None else '*'
+        for j in iglobpath(_glob, sort_result=glob_sorted):
+            j = os.path.basename(j)
+            if subsequence_match(j, nextone, csc):
+                out.add((i or ()) + (j, ))
+    return out
+
+
+def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     """Completes based on a path name."""
     # string stuff for automatic quoting
     path_str_start = ''
@@ -165,22 +248,48 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True):
         path_str_end = p[3]
     tilde = '~'
     paths = set()
-    csc = builtins.__xonsh_env__.get('CASE_SENSITIVE_COMPLETIONS')
-    for s in iglobpath(prefix + '*', ignore_case=(not csc)):
+    env = builtins.__xonsh_env__
+    csc = env.get('CASE_SENSITIVE_COMPLETIONS')
+    glob_sorted = env.get('GLOB_SORTED')
+    for s in iglobpath(prefix + '*', ignore_case=(not csc),
+                       sort_result=glob_sorted):
         paths.add(s)
+    if len(paths) == 0 and env.get('SUBSEQUENCE_PATH_COMPLETION'):
+        # this block implements 'subsequence' matching, similar to fish and zsh.
+        # matches are based on subsequences, not substrings.
+        # e.g., ~/u/ro completes to ~/lou/carcolh
+        # see above functions for details.
+        p = _splitpath(os.path.expanduser(prefix))
+        if len(p) != 0:
+            if p[0] == '':
+                basedir = ('', )
+                p = p[1:]
+            else:
+                basedir = None
+            matches_so_far = {basedir}
+            for i in p:
+                matches_so_far = _expand_one(matches_so_far, i, csc)
+            paths |= {_joinpath(i) for i in matches_so_far}
+    if len(paths) == 0 and env.get('FUZZY_PATH_COMPLETION'):
+        threshold = env.get('SUGGEST_THRESHOLD')
+        for s in iglobpath(os.path.dirname(prefix) + '*', ignore_case=(not csc),
+                           sort_result=glob_sorted):
+            if levenshtein(prefix, s, threshold) < threshold:
+                paths.add(s)
     if tilde in prefix:
         home = os.path.expanduser(tilde)
         paths = {s.replace(home, tilde) for s in paths}
     if cdpath:
         _add_cdpaths(paths, prefix)
+    paths = set(filter(filtfunc, paths))
     paths = _quote_paths({_normpath(s) for s in paths},
                          path_str_start,
                          path_str_end)
-    _add_env(paths, prefix)
-    _add_dots(paths, prefix)
+    paths.update(filter(filtfunc, _dots(prefix)))
+    paths.update(filter(filtfunc, _env(prefix)))
     return paths, lprefix
 
 
 def complete_dir(prefix, line, start, end, ctx, cdpath=False):
-    o, lp = complete_path(prefix, line, start, end, cdpath)
-    return {i for i in o if os.path.isdir(i)}, lp
+    return complete_path(prefix, line, start, end, cdpath,
+                         filtfunc=os.path.isdir)
