@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Tools for inspecting Python objects.
 
 This file was forked from the IPython project:
@@ -7,51 +8,54 @@ This file was forked from the IPython project:
 * Copyright (c) 2001, Janko Hauser <jhauser@zscout.de>
 * Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
 """
-from __future__ import print_function, unicode_literals
-
 import os
+import io
 import sys
 import types
 import inspect
+import itertools
 import linecache
-import io as stdlib_io
+import importlib
+import collections
 
-from xonsh import openpy
-from xonsh.tools import cast_unicode, safe_hasattr, string_types, indent
-
-if sys.version_info[0] > 2:
-    ISPY3K = True
-    from itertools import zip_longest
-else:
-    ISPY3K = False
-    from itertools import izip_longest as zip_longest
+from xonsh.lazyasd import LazyObject
+from xonsh.tokenize import detect_encoding
+from xonsh.openpy import read_py_file
+from xonsh.tools import (cast_unicode, safe_hasattr, indent,
+                         print_color, format_color)
+from xonsh.platform import HAS_PYGMENTS, PYTHON_VERSION_INFO
+from xonsh.lazyimps import pygments, pyghooks
 
 
 # builtin docstrings to ignore
-_func_call_docstring = types.FunctionType.__call__.__doc__
-_object_init_docstring = object.__init__.__doc__
-_builtin_type_docstrings = {
+_func_call_docstring = LazyObject(lambda: types.FunctionType.__call__.__doc__,
+                                  globals(), '_func_call_docstring')
+_object_init_docstring = LazyObject(lambda: object.__init__.__doc__,
+                                    globals(), '_object_init_docstring')
+_builtin_type_docstrings = LazyObject(lambda: {
     t.__doc__
     for t in (types.ModuleType, types.MethodType, types.FunctionType)
-}
+    }, globals(), '_builtin_type_docstrings')
 
-_builtin_func_type = type(all)
-_builtin_meth_type = type(
-    str.upper)  # Bound methods have the same type as builtin functions
+_builtin_func_type = LazyObject(lambda: type(all), globals(),
+                                '_builtin_func_type')
+# Bound methods have the same type as builtin functions
+_builtin_meth_type = LazyObject(lambda: type(str.upper), globals(),
+                                '_builtin_meth_type')
 
-info_fields = [
+info_fields = LazyObject(lambda: [
     'type_name', 'base_class', 'string_form', 'namespace', 'length', 'file',
     'definition', 'docstring', 'source', 'init_definition', 'class_docstring',
     'init_docstring', 'call_def', 'call_docstring',
     # These won't be printed but will be used to determine how to
     # format the object
     'ismagic', 'isalias', 'isclass', 'argspec', 'found', 'name'
-]
+    ], globals(), 'info_fields')
 
 
 def object_info(**kw):
     """Make an object info dict with all fields present."""
-    infodict = dict(zip_longest(info_fields, [None]))
+    infodict = dict(itertools.zip_longest(info_fields, [None]))
     infodict.update(kw)
     return infodict
 
@@ -75,8 +79,8 @@ def get_encoding(obj):
         # Print only text files, not extension binaries.  Note that
         # getsourcelines returns lineno with 1-offset and page() uses
         # 0-offset, so we must adjust.
-        with stdlib_io.open(ofile, 'rb') as buf:  # Tweaked to use io.open for Python 2
-            encoding, _ = openpy.detect_encoding(buf.readline)
+        with io.open(ofile, 'rb') as buf:  # Tweaked to use io.open for Python 2
+            encoding, _ = detect_encoding(buf.readline)
         return encoding
 
 
@@ -95,7 +99,7 @@ def getdoc(obj):
         pass
     else:
         # if we get extra info, we add it to the normal docstring.
-        if isinstance(ds, string_types):
+        if isinstance(ds, str):
             return inspect.cleandoc(ds)
 
     try:
@@ -157,7 +161,7 @@ def getargspec(obj):
     if safe_hasattr(obj, '__call__') and not is_simple_callable(obj):
         obj = obj.__call__
 
-    return inspect.getfullargspec(obj) if ISPY3K else inspect.getargspec(obj)
+    return inspect.getfullargspec(obj)
 
 
 def format_argspec(argspec):
@@ -295,6 +299,17 @@ def find_source_lines(obj):
     return lineno
 
 
+if PYTHON_VERSION_INFO < (3, 5, 0):
+    FrameInfo = collections.namedtuple('FrameInfo', ['frame', 'filename',
+                                                     'lineno', 'function',
+                                                     'code_context', 'index'])
+    def getouterframes(frame, context=1):
+        """Wrapper for getouterframes so that it acts like the Python v3.5 version."""
+        return [FrameInfo(*f) for f in inspect.getouterframes(frame, context=context)]
+else:
+    getouterframes = inspect.getouterframes
+
+
 class Inspector(object):
     """Inspects objects."""
 
@@ -336,8 +351,6 @@ class Inspector(object):
         if inspect.isclass(obj):
             header = self.__head('Class constructor information:\n')
             obj = obj.__init__
-        elif (not ISPY3K) and type(obj) is types.InstanceType:
-            obj = obj.__call__
 
         output = self._getdef(obj, oname)
         if output is None:
@@ -406,11 +419,11 @@ class Inspector(object):
             # Print only text files, not extension binaries.  Note that
             # getsourcelines returns lineno with 1-offset and page() uses
             # 0-offset, so we must adjust.
-            o = openpy.read_py_file(ofile, skip_encoding_cookie=False)
+            o = read_py_file(ofile, skip_encoding_cookie=False)
             print(o, lineno - 1)
 
-    def _format_fields(self, fields, title_width=0):
-        """Formats a list of fields for display.
+    def _format_fields_str(self, fields, title_width=0):
+        """Formats a list of fields for display using color strings.
 
         Parameters
         ----------
@@ -423,12 +436,61 @@ class Inspector(object):
         if title_width == 0:
             title_width = max(len(title) + 2 for title, _ in fields)
         for title, content in fields:
+            title_len = len(title)
+            title = '{BOLD_RED}' + title + ':{NO_COLOR}'
             if len(content.splitlines()) > 1:
-                title = title + ":\n"
+                title += '\n'
             else:
-                title = (title + ":").ljust(title_width)
+                title += " ".ljust(title_width - title_len)
             out.append(cast_unicode(title) + cast_unicode(content))
-        return "\n".join(out)
+        return format_color("\n".join(out) + '\n')
+
+    def _format_fields_tokens(self, fields, title_width=0):
+        """Formats a list of fields for display using color tokens from
+        pygments.
+
+        Parameters
+        ----------
+        fields : list
+          A list of 2-tuples: (field_title, field_content)
+        title_width : int
+          How many characters to pad titles to. Default to longest title.
+        """
+        out = []
+        if title_width == 0:
+            title_width = max(len(title) + 2 for title, _ in fields)
+        for title, content in fields:
+            title_len = len(title)
+            title = '{BOLD_RED}' + title + ':{NO_COLOR}'
+            if not isinstance(content, str) or len(content.splitlines()) > 1:
+                title += '\n'
+            else:
+                title += " ".ljust(title_width - title_len)
+            out += pyghooks.partial_color_tokenize(title)
+            if isinstance(content, str):
+                out[-1] = (out[-1][0], out[-1][1] + content + '\n')
+            else:
+                out += content
+                out[-1] = (out[-1][0], out[-1][1] + '\n')
+        out[-1] = (out[-1][0], out[-1][1] + '\n')
+        return out
+
+    def _format_fields(self, fields, title_width=0):
+        """Formats a list of fields for display using color tokens from
+        pygments.
+
+        Parameters
+        ----------
+        fields : list
+          A list of 2-tuples: (field_title, field_content)
+        title_width : int
+          How many characters to pad titles to. Default to longest title.
+        """
+        if HAS_PYGMENTS:
+            rtn = self._format_fields_tokens(fields, title_width=title_width)
+        else:
+            rtn = self._format_fields_str(fields, title_width=title_width)
+        return rtn
 
     # The fields to be displayed by pinfo: (fancy_name, key_in_info_dict)
     pinfo_fields1 = [("Type", "type_name")]
@@ -471,14 +533,6 @@ class Inspector(object):
                     displayfields.append((title, field.rstrip()))
 
         add_fields(self.pinfo_fields1)
-
-        # Base class for old-style instances
-        if ((not ISPY3K) and
-           isinstance(obj, types.InstanceType) and
-           info['base_class']):
-                o = ("Base Class", info['base_class'].rstrip())
-                displayfields.append(o)
-
         add_fields(self.pinfo_fields2)
 
         # Namespace
@@ -510,7 +564,7 @@ class Inspector(object):
 
         # Finally send to printer/pager:
         if displayfields:
-            print(self._format_fields(displayfields))
+            print_color(self._format_fields(displayfields))
 
     def info(self, obj, oname='', info=None, detail_level=0):
         """Compute a dict with detailed information about an object.
@@ -536,7 +590,7 @@ class Inspector(object):
         # Get docstring, special-casing aliases:
         if isalias:
             if not callable(obj):
-                if len(obj) >= 2 and isinstance(obj[1], string_types):
+                if len(obj) >= 2 and isinstance(obj[1], str):
                     ds = "Alias to the system command:\n  {0}".format(obj[1])
                 else:  # pylint:disable=bare-except
                     ds = "Alias: " + str(obj)
@@ -624,7 +678,11 @@ class Inspector(object):
                     if hasattr(obj, '__class__'):
                         source = getsource(obj.__class__, binary_file)
                 if source is not None:
-                    out['source'] = source.rstrip()
+                    source = source.rstrip()
+                    if HAS_PYGMENTS:
+                        lexer = pyghooks.XonshLexer()
+                        source = list(pygments.lex(source, lexer=lexer))
+                    out['source'] = source
             except Exception:  # pylint:disable=broad-except
                 pass
 
