@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Interface for running Python functions as subprocess-mode commands.
 
 Code for several helper methods in the `ProcProxy` class have been reproduced
@@ -10,44 +11,58 @@ import io
 import os
 import sys
 import time
-from threading import Thread
-from collections import Sequence
-from subprocess import Popen, PIPE, DEVNULL, STDOUT, TimeoutExpired
+import signal
+import builtins
+import functools
+import threading
+import importlib
+import subprocess
+import collections
+import collections.abc as abc
 
-from xonsh.tools import redirect_stdout, redirect_stderr, ON_WINDOWS, ON_LINUX, \
-    fallback, print_exception
-
-if ON_LINUX:
-    from xonsh.teepty import TeePTY
-else:
-    TeePTY = None
-
-if ON_WINDOWS:
-    import _winapi
-    import msvcrt
-
-    class Handle(int):
-        closed = False
-
-        def Close(self, CloseHandle=_winapi.CloseHandle):
-            if not self.closed:
-                self.closed = True
-                CloseHandle(self)
-
-        def Detach(self):
-            if not self.closed:
-                self.closed = True
-                return int(self)
-            raise ValueError("already closed")
-
-        def __repr__(self):
-            return "Handle(%d)" % int(self)
-
-        __del__ = Close
-        __str__ = __repr__
+from xonsh.platform import ON_WINDOWS, ON_LINUX, ON_POSIX
+from xonsh.tools import (redirect_stdout, redirect_stderr, fallback,
+                         print_exception, XonshCalledProcessError)
+from xonsh.teepty import TeePTY
+from xonsh.lazyasd import LazyObject, lazyobject
 
 
-class ProcProxy(Thread):
+# force some lazy imports so we don't have errors on non-windows platforms
+_winapi = LazyObject(lambda: importlib.import_module('_winapi'),
+                     globals(), '_winapi')
+
+@lazyobject
+def msvcrt():
+    if ON_WINDOWS:
+        import msvcrt as m
+    else:
+        m = None
+    return m
+
+
+class Handle(int):
+    closed = False
+
+    def Close(self, CloseHandle=None):
+        CloseHandle = CloseHandle or _winapi.CloseHandle
+        if not self.closed:
+            self.closed = True
+            CloseHandle(self)
+
+    def Detach(self):
+        if not self.closed:
+            self.closed = True
+            return int(self)
+        raise ValueError("already closed")
+
+    def __repr__(self):
+        return "Handle(%d)" % int(self)
+
+    __del__ = Close
+    __str__ = __repr__
+
+
+class ProcProxy(threading.Thread):
     """
     Class representing a function to be run as a subprocess-mode command.
     """
@@ -135,7 +150,7 @@ class ProcProxy(Thread):
             if universal_newlines:
                 self.stderr = io.TextIOWrapper(self.stderr)
 
-        Thread.__init__(self)
+        super().__init__()
         self.start()
 
     def run(self):
@@ -173,9 +188,10 @@ class ProcProxy(Thread):
     def poll(self):
         """Check if the function has completed.
 
-        :return: `None` if the function is still executing, `True` if the
-                 function finished successfully, and `False` if there was an
-                 error
+        Returns
+        -------
+        `None` if the function is still executing, `True` if the function
+        finished successfully, and `False` if there was an error.
         """
         return self.returncode
 
@@ -212,10 +228,10 @@ class ProcProxy(Thread):
                     p2cread, _ = _winapi.CreatePipe(None, 0)
                     p2cread = Handle(p2cread)
                     _winapi.CloseHandle(_)
-            elif stdin == PIPE:
+            elif stdin == subprocess.PIPE:
                 p2cread, p2cwrite = _winapi.CreatePipe(None, 0)
                 p2cread, p2cwrite = Handle(p2cread), Handle(p2cwrite)
-            elif stdin == DEVNULL:
+            elif stdin == subprocess.DEVNULL:
                 p2cread = msvcrt.get_osfhandle(self._get_devnull())
             elif isinstance(stdin, int):
                 p2cread = msvcrt.get_osfhandle(stdin)
@@ -230,10 +246,10 @@ class ProcProxy(Thread):
                     _, c2pwrite = _winapi.CreatePipe(None, 0)
                     c2pwrite = Handle(c2pwrite)
                     _winapi.CloseHandle(_)
-            elif stdout == PIPE:
+            elif stdout == subprocess.PIPE:
                 c2pread, c2pwrite = _winapi.CreatePipe(None, 0)
                 c2pread, c2pwrite = Handle(c2pread), Handle(c2pwrite)
-            elif stdout == DEVNULL:
+            elif stdout == subprocess.DEVNULL:
                 c2pwrite = msvcrt.get_osfhandle(self._get_devnull())
             elif isinstance(stdout, int):
                 c2pwrite = msvcrt.get_osfhandle(stdout)
@@ -248,12 +264,12 @@ class ProcProxy(Thread):
                     _, errwrite = _winapi.CreatePipe(None, 0)
                     errwrite = Handle(errwrite)
                     _winapi.CloseHandle(_)
-            elif stderr == PIPE:
+            elif stderr == subprocess.PIPE:
                 errread, errwrite = _winapi.CreatePipe(None, 0)
                 errread, errwrite = Handle(errread), Handle(errwrite)
-            elif stderr == STDOUT:
+            elif stderr == subprocess.STDOUT:
                 errwrite = c2pwrite
-            elif stderr == DEVNULL:
+            elif stderr == subprocess.DEVNULL:
                 errwrite = msvcrt.get_osfhandle(self._get_devnull())
             elif isinstance(stderr, int):
                 errwrite = msvcrt.get_osfhandle(stderr)
@@ -278,9 +294,9 @@ class ProcProxy(Thread):
 
             if stdin is None:
                 pass
-            elif stdin == PIPE:
+            elif stdin == subprocess.PIPE:
                 p2cread, p2cwrite = os.pipe()
-            elif stdin == DEVNULL:
+            elif stdin == subprocess.DEVNULL:
                 p2cread = self._get_devnull()
             elif isinstance(stdin, int):
                 p2cread = stdin
@@ -290,9 +306,9 @@ class ProcProxy(Thread):
 
             if stdout is None:
                 pass
-            elif stdout == PIPE:
+            elif stdout == subprocess.PIPE:
                 c2pread, c2pwrite = os.pipe()
-            elif stdout == DEVNULL:
+            elif stdout == subprocess.DEVNULL:
                 c2pwrite = self._get_devnull()
             elif isinstance(stdout, int):
                 c2pwrite = stdout
@@ -302,11 +318,11 @@ class ProcProxy(Thread):
 
             if stderr is None:
                 pass
-            elif stderr == PIPE:
+            elif stderr == subprocess.PIPE:
                 errread, errwrite = os.pipe()
-            elif stderr == STDOUT:
+            elif stderr == subprocess.STDOUT:
                 errwrite = c2pwrite
-            elif stderr == DEVNULL:
+            elif stderr == subprocess.DEVNULL:
                 errwrite = self._get_devnull()
             elif isinstance(stderr, int):
                 errwrite = stderr
@@ -319,6 +335,38 @@ class ProcProxy(Thread):
                     errread, errwrite)
 
 
+def wrap_simple_command(f, args, stdin, stdout, stderr):
+    """Decorator for creating 'simple' callable aliases."""
+    bgable = getattr(f, '__xonsh_backgroundable__', True)
+    @functools.wraps(f)
+    def wrapped_simple_command(args, stdin, stdout, stderr):
+        try:
+            i = stdin.read()
+            if bgable:
+                with redirect_stdout(stdout), redirect_stderr(stderr):
+                    r = f(args, i)
+            else:
+                r = f(args, i)
+
+            cmd_result = 0
+            if isinstance(r, str):
+                stdout.write(r)
+            elif isinstance(r, abc.Sequence):
+                if r[0] is not None:
+                    stdout.write(r[0])
+                if r[1] is not None:
+                    stderr.write(r[1])
+                if len(r) > 2 and r[2] is not None:
+                    cmd_result = r[2]
+            elif r is not None:
+                stdout.write(str(r))
+            return cmd_result
+        except Exception:
+            print_exception()
+            return 1  # returncode for failure
+    return wrapped_simple_command
+
+
 class SimpleProcProxy(ProcProxy):
     """Variant of `ProcProxy` for simpler functions.
 
@@ -329,36 +377,86 @@ class SimpleProcProxy(ProcProxy):
 
     def __init__(self, f, args, stdin=None, stdout=None, stderr=None,
                  universal_newlines=False):
-        def wrapped_simple_command(args, stdin, stdout, stderr):
-            try:
-                i = stdin.read()
-                with redirect_stdout(stdout), redirect_stderr(stderr):
-                    r = f(args, i)
-                if isinstance(r, str):
-                    stdout.write(r)
-                elif isinstance(r, Sequence):
-                    if r[0] is not None:
-                        stdout.write(r[0])
-                    if r[1] is not None:
-                        stderr.write(r[1])
-                elif r is not None:
-                    stdout.write(str(r))
-                return 0  # returncode for succees
-            except Exception:
-                print_exception()
-                return 1  # returncode for failure
-        super().__init__(wrapped_simple_command,
-                         args, stdin, stdout, stderr,
-                         universal_newlines)
+        f = wrap_simple_command(f, args, stdin, stdout, stderr)
+        super().__init__(f, args, stdin, stdout, stderr, universal_newlines)
+
+#
+# Foreground Process Proxies
+#
+
+class ForegroundProcProxy(object):
+    """This is process proxy class that runs its alias functions on the
+    same thread that it was called from, which is typically the main thread.
+    This prevents backgrounding the process, but enables debugger and
+    profiler tools (functions) be run on the same thread that they are
+    attempting to debug.
+    """
+
+    def __init__(self, f, args, stdin=None, stdout=None, stderr=None,
+                 universal_newlines=False):
+        self.f = f
+        self.args = args
+        self.pid = os.getpid()
+        self.returncode = None
+        self.stdin = stdin
+        self.stdout = None
+        self.stderr = None
+        self.universal_newlines = universal_newlines
+
+    def poll(self):
+        """Check if the function has completed via the returncode or None.
+        """
+        return self.returncode
+
+    def wait(self, timeout=None):
+        """Runs the function and returns the result. Timeout argument only
+        present for API compatability.
+        """
+        if self.f is None:
+            return
+        if self.stdin is None:
+            stdin = io.StringIO("")
+        else:
+            stdin = io.TextIOWrapper(self.stdin)
+        r = self.f(self.args, stdin, self.stdout, self.stderr)
+        self.returncode = 0 if r is None else r
+        return self.returncode
 
 
-@fallback(ON_LINUX, Popen)
+class SimpleForegroundProcProxy(ForegroundProcProxy):
+    """Variant of `ForegroundProcProxy` for simpler functions.
+
+    The function passed into the initializer for `SimpleForegroundProcProxy`
+    should have the form described in the xonsh tutorial. This function is
+    then wrapped to make a new function of the form expected by
+    `ForegroundProcProxy`.
+    """
+
+    def __init__(self, f, args, stdin=None, stdout=None, stderr=None,
+                 universal_newlines=False):
+        f = wrap_simple_command(f, args, stdin, stdout, stderr)
+        super().__init__(f, args, stdin, stdout, stderr, universal_newlines)
+
+
+def foreground(f):
+    """Decorator that specifies that a callable alias should be run only
+    as a foreground process. This is often needed for debuggers and profilers.
+    """
+    f.__xonsh_backgroundable__ = False
+    return f
+
+#
+# Pseudo-terminal Proxies
+#
+
+
+@fallback(ON_LINUX, subprocess.Popen)
 class TeePTYProc(object):
 
     def __init__(self, args, stdin=None, stdout=None, stderr=None, preexec_fn=None,
                  env=None, universal_newlines=False):
-        """Popen replacement for running commands in teed psuedo-terminal. This 
-        allows the capturing AND streaming of stdout and stderr.  Availability 
+        """Popen replacement for running commands in teed psuedo-terminal. This
+        allows the capturing AND streaming of stdout and stderr.  Availability
         is Linux-only.
         """
         self.stdin = stdin
@@ -366,11 +464,20 @@ class TeePTYProc(object):
         self._stderr = stderr
         self.args = args
         self.universal_newlines = universal_newlines
-        
-        self._tpty = tpty = TeePTY()
+        xenv = builtins.__xonsh_env__ if hasattr(builtins, '__xonsh_env__') \
+                                      else {'XONSH_ENCODING': 'utf-8',
+                                            'XONSH_ENCODING_ERRORS': 'strict'}
+
+        if not os.access(args[0], os.F_OK):
+            raise FileNotFoundError('command {0!r} not found'.format(args[0]))
+        elif not os.access(args[0], os.X_OK) or os.path.isdir(args[0]):
+            raise PermissionError('permission denied: {0!r}'.format(args[0]))
+        self._tpty = tpty = TeePTY(encoding=xenv.get('XONSH_ENCODING'),
+                                   errors=xenv.get('XONSH_ENCODING_ERRORS'))
         if preexec_fn is not None:
             preexec_fn()
-        tpty.spawn(args, env=env, stdin=stdin)
+        delay = xenv.get('TEEPTY_PIPE_DELAY')
+        tpty.spawn(args, env=env, stdin=stdin, delay=delay)
 
     @property
     def pid(self):
@@ -379,21 +486,37 @@ class TeePTYProc(object):
 
     @property
     def returncode(self):
-        """The return code of the spawned process."""
-        return self._tpty.returncode
+        """The return value of the spawned process or None if the process
+        exited due to a signal."""
+        if os.WIFEXITED(self._tpty.wcode):
+            return os.WEXITSTATUS(self._tpty.wcode)
+        else:
+            return None
+
+    @property
+    def signal(self):
+        """If the process was terminated by a signal a 2-tuple is returned
+        containing the signal number and a boolean indicating whether a core
+        file was produced. Otherwise None is returned."""
+        if os.WIFSIGNALED(self._tpty.wcode):
+            return (os.WTERMSIG(self._tpty.wcode),
+                    os.WCOREDUMP(self._tpty.wcode))
+        else:
+            return None
 
     def poll(self):
-        """Polls the spawned process and returns the returncode."""
-        return self._tpty.returncode
+        """Polls the spawned process and returns the os.wait code."""
+        return _wcode_to_popen(self._tpty.wcode)
 
     def wait(self, timeout=None):
-        """Waits for the spawned process to finish, up to a timeout."""
+        """Waits for the spawned process to finish, up to a timeout.
+        Returns the return os.wait code."""
         tpty = self._tpty
         t0 = time.time()
-        while tpty.returncode is None:
+        while tpty.wcode is None:
             if timeout is not None and timeout < (time.time() - t0):
-                raise TimeoutExpired
-        return tpty.returncode
+                raise subprocess.TimeoutExpired
+        return _wcode_to_popen(tpty.wcode)
 
     @property
     def stdout(self):
@@ -407,3 +530,99 @@ class TeePTYProc(object):
         else:
             self._stdout = self._tpty.buffer
         return self._stdout
+
+
+def _wcode_to_popen(code):
+    """Converts os.wait return code into Popen format."""
+    if os.WIFEXITED(code):
+        return os.WEXITSTATUS(code)
+    elif os.WIFSIGNALED(code):
+        return -1 * os.WTERMSIG(code)
+    else:
+        # Can this happen? Let's find out. Returning None is not an option.
+        raise ValueError("Invalid os.wait code: {}".format(code))
+
+
+_CCTuple = collections.namedtuple("_CCTuple", ["stdin", "stdout", "stderr",
+                "pid", "returncode", "args", "alias", "stdin_redirect",
+                "stdout_redirect", "stderr_redirect", "timestamp",
+                "executed_cmd"])
+
+
+class CompletedCommand(_CCTuple):
+    """Represents a completed subprocess-mode command."""
+
+    def __bool__(self):
+        return self.returncode == 0
+
+    def __iter__(self):
+        if not self.stdout:
+            raise StopIteration()
+
+        pre = self.stdout
+        post = None
+
+        while post != '':
+            pre, sep, post = pre.partition('\n')
+            # this line may be optional since we use universal newlines.
+            pre = pre[:-1] if pre and pre[-1] == '\r' else pre
+            yield pre
+            pre = post
+
+
+    def itercheck(self):
+        yield from self
+        if self.returncode:
+            # I included self, as providing access to stderr and other details
+            # useful when instance isn't assigned to a variable in the shell.
+            raise XonshCalledProcessError(self.returncode, self.executed_cmd,
+                                          self.stdout, self.stderr, self)
+
+    @property
+    def inp(self):
+        """Creates normalized input string from args."""
+        return ' '.join(self.args)
+
+    @property
+    def out(self):
+        """Alias to stdout."""
+        return self.stdout
+
+    @property
+    def err(self):
+        """Alias to stderr."""
+        return self.stderr
+
+    @property
+    def rtn(self):
+        """Alias to return code."""
+        return self.returncode
+
+CompletedCommand.__new__.__defaults__ = (None,) * len(CompletedCommand._fields)
+
+
+class HiddenCompletedCommand(CompletedCommand):
+    def __repr__(self):
+        return ''
+
+
+def pause_call_resume(p, f, *args, **kwargs):
+    """For a process p, this will call a function f with the remaining args and
+    and kwargs. If the process cannot accept signals, the function will be called.
+
+    Parameters
+    ----------
+    p : Popen object or similar
+    f : callable
+    args : remaining arguments
+    kwargs : keyword arguments
+    """
+    can_send_signal = hasattr(p, 'send_signal') and ON_POSIX
+    if can_send_signal:
+        p.send_signal(signal.SIGSTOP)
+    try:
+        f(*args, **kwargs)
+    except Exception:
+        pass
+    if can_send_signal:
+        p.send_signal(signal.SIGCONT)
