@@ -3,32 +3,35 @@
 
 Portions of this code related to initializing the readline library
 are included from the IPython project.  The IPython project is:
-    * Copyright (c) 2008-2014, IPython Development Team
-    * Copyright (c) 2001-2007, Fernando Perez <fernando.perez@colorado.edu>
-    * Copyright (c) 2001, Janko Hauser <jhauser@zscout.de>
-    * Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
+
+* Copyright (c) 2008-2014, IPython Development Team
+* Copyright (c) 2001-2007, Fernando Perez <fernando.perez@colorado.edu>
+* Copyright (c) 2001, Janko Hauser <jhauser@zscout.de>
+* Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
+
 """
 import os
 import sys
+import cmd
 import time
 import select
 import builtins
 import importlib
-from cmd import Cmd
-from threading import Thread
-from collections import deque
+import threading
+import collections
 
-from xonsh import lazyjson
+from xonsh.lazyjson import LazyJSON
+from xonsh.lazyasd import LazyObject
 from xonsh.base_shell import BaseShell
-from xonsh.ansi_colors import partial_color_format, color_style_names, color_style
+from xonsh.ansi_colors import ansi_partial_color_format, ansi_color_style_names, ansi_color_style
 from xonsh.environ import partial_format_prompt, multiline_prompt
 from xonsh.tools import print_exception
-from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS, ON_CYGWIN, ON_DARWIN
+from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_DARWIN
+from xonsh.lazyimps import pygments, pyghooks
 
-if HAS_PYGMENTS:
-    from xonsh import pyghooks
-    import pygments
-    from pygments.formatters.terminal256 import Terminal256Formatter
+terminal256 = LazyObject(lambda: importlib.import_module(
+                                    'pygments.formatters.terminal256'),
+                      globals(), 'terminal')
 
 readline = None
 RL_COMPLETION_SUPPRESS_APPEND = RL_LIB = RL_STATE = None
@@ -38,6 +41,9 @@ RL_VARIABLE_VALUE = None
 _RL_STATE_DONE = 0x1000000
 _RL_STATE_ISEARCH = 0x0000080
 
+_RL_PREV_CASE_SENSITIVE_COMPLETIONS = 'to-be-set'
+
+
 def setup_readline():
     """Sets up the readline module and completion suppression, if available."""
     global RL_COMPLETION_SUPPRESS_APPEND, RL_LIB, RL_CAN_RESIZE, RL_STATE, readline
@@ -69,7 +75,7 @@ def setup_readline():
             RL_COMPLETION_SUPPRESS_APPEND = None
         try:
             RL_STATE = ctypes.c_int.in_dll(lib, 'rl_readline_state')
-        except:
+        except Exception:
             pass
         RL_CAN_RESIZE = hasattr(lib, 'rl_reset_screen_size')
     env = builtins.__xonsh_env__
@@ -124,6 +130,20 @@ def teardown_readline():
         return
 
 
+def _rebind_case_sensitive_completions():
+    # handle case sensitive, see Github issue #1342 for details
+    global _RL_PREV_CASE_SENSITIVE_COMPLETIONS
+    env = builtins.__xonsh_env__
+    case_sensitive = env.get('CASE_SENSITIVE_COMPLETIONS')
+    if case_sensitive is _RL_PREV_CASE_SENSITIVE_COMPLETIONS:
+        return
+    if case_sensitive:
+        readline.parse_and_bind("set completion-ignore-case off")
+    else:
+        readline.parse_and_bind("set completion-ignore-case on")
+    _RL_PREV_CASE_SENSITIVE_COMPLETIONS = case_sensitive
+
+
 def fix_readline_state_after_ctrl_c():
     """
     Fix to allow Ctrl-C to exit reverse-i-search.
@@ -137,7 +157,7 @@ def fix_readline_state_after_ctrl_c():
             _q = readline.rl.mode.process_keyevent_queue
             if len(_q) > 1:
                 _q.pop()
-        except:
+        except Exception:
             pass
     if RL_STATE is None:
         return
@@ -186,10 +206,12 @@ def _insert_text_func(s, readline):
     return inserter
 
 
-DEDENT_TOKENS = frozenset(['raise', 'return', 'pass', 'break', 'continue'])
+DEDENT_TOKENS = LazyObject(lambda: frozenset(['raise', 'return', 'pass',
+                                              'break', 'continue']),
+                           globals(), 'DEDENT_TOKENS')
 
 
-class ReadlineShell(BaseShell, Cmd):
+class ReadlineShell(BaseShell, cmd.Cmd):
     """The readline based xonsh shell."""
 
     def __init__(self, completekey='tab', stdin=None, stdout=None, **kwargs):
@@ -201,7 +223,7 @@ class ReadlineShell(BaseShell, Cmd):
         self._current_indent = ''
         self._current_prompt = ''
         self._force_hide = None
-        self.cmdqueue = deque()
+        self.cmdqueue = collections.deque()
 
     def __del__(self):
         teardown_readline()
@@ -229,12 +251,17 @@ class ReadlineShell(BaseShell, Cmd):
     def completedefault(self, text, line, begidx, endidx):
         """Implements tab-completion for text."""
         rl_completion_suppress_append()  # this needs to be called each time
+        _rebind_case_sensitive_completions()
+
         mline = line.partition(' ')[2]
         offs = len(mline) - len(text)
-        x = [(i[offs:] if " " in i[:-1] else i)
-             for i in self.completer.complete(text, line,
-                                              begidx, endidx,
-                                              ctx=self.ctx)[0]]
+        if self.completer is None:
+            x = []
+        else:
+            x = [(i[offs:] if " " in i[:-1] else i)
+                 for i in self.completer.complete(text, line,
+                                                  begidx, endidx,
+                                                  ctx=self.ctx)[0]]
         return x
 
     # tab complete on first index too
@@ -390,8 +417,8 @@ class ReadlineShell(BaseShell, Cmd):
         except Exception:  # pylint: disable=broad-except
             print_exception()
         hide = True if self._force_hide is None else self._force_hide
-        p = partial_color_format(p, style=env.get('XONSH_COLOR_STYLE'),
-                                 hide=hide)
+        p = ansi_partial_color_format(p, style=env.get('XONSH_COLOR_STYLE'),
+                                      hide=hide)
         self._current_prompt = p
         self.settitle()
         return p
@@ -401,7 +428,7 @@ class ReadlineShell(BaseShell, Cmd):
         codes.
         """
         hide = hide if self._force_hide is None else self._force_hide
-        return partial_color_format(string, hide=hide,
+        return ansi_partial_color_format(string, hide=hide,
                     style=builtins.__xonsh_env__.get('XONSH_COLOR_STYLE'))
 
     def print_color(self, string, hide=False, **kwargs):
@@ -412,21 +439,21 @@ class ReadlineShell(BaseShell, Cmd):
             env = builtins.__xonsh_env__
             self.styler.style_name = env.get('XONSH_COLOR_STYLE')
             style_proxy = pyghooks.xonsh_style_proxy(self.styler)
-            formatter = Terminal256Formatter(style=style_proxy)
+            formatter = terminal256.Terminal256Formatter(style=style_proxy)
             s = pygments.format(string, formatter).rstrip()
         print(s, **kwargs)
 
     def color_style_names(self):
         """Returns an iterable of all available style names."""
-        return color_style_names()
+        return ansi_color_style_names()
 
     def color_style(self):
         """Returns the current color map."""
         style = style=builtins.__xonsh_env__.get('XONSH_COLOR_STYLE')
-        return color_style(style=style)
+        return ansi_color_style(style=style)
 
 
-class ReadlineHistoryAdder(Thread):
+class ReadlineHistoryAdder(threading.Thread):
 
     def __init__(self, wait_for_gc=True, *args, **kwargs):
         """Thread responsible for adding inputs from history to the current readline
@@ -449,7 +476,7 @@ class ReadlineHistoryAdder(Thread):
         i = 1
         for _, _, f in files:
             try:
-                lj = lazyjson.LazyJSON(f, reopen=False)
+                lj = LazyJSON(f, reopen=False)
                 for cmd in lj['cmds']:
                     inp = cmd['inp'].splitlines()
                     for line in inp:
@@ -460,5 +487,5 @@ class ReadlineHistoryAdder(Thread):
                             RL_LIB.history_set_pos(i)
                         i += 1
                 lj.close()
-            except (IOError, OSError):
+            except (IOError, OSError, ValueError):
                 continue
