@@ -3,25 +3,22 @@ import os
 import re
 import sys
 import inspect
+import argparse
 import linecache
 import importlib
-from functools import lru_cache
-from argparse import ArgumentParser
+import functools
 
 from xonsh.lazyasd import LazyObject
 from xonsh.platform import HAS_PYGMENTS
 from xonsh.tools import DefaultNotGiven, print_color, normabspath, to_bool
 from xonsh.inspectors import find_file, getouterframes
 from xonsh.environ import _replace_home
+from xonsh.lazyimps import pygments, pyghooks
 
 
-pygments = LazyObject(lambda: importlib.import_module('pygments'),
-                      globals(), 'pygments')
 terminal = LazyObject(lambda: importlib.import_module(
                                 'pygments.formatters.terminal'),
                       globals(), 'terminal')
-pyghooks = LazyObject(lambda: importlib.import_module('xonsh.pyghooks'),
-                      globals(), 'pyghooks')
 
 class TracerType(object):
     """Represents a xonsh tracer object, which keeps track of all tracing
@@ -161,10 +158,10 @@ def _color(ns, args):
     tracer.usecolor = ns.toggle
 
 
-@lru_cache(1)
+@functools.lru_cache(1)
 def _tracer_create_parser():
     """Creates tracer argument parser"""
-    p = ArgumentParser(prog='trace',
+    p = argparse.ArgumentParser(prog='trace',
                        description='tool for tracing xonsh code as it runs.')
     subp = p.add_subparsers(title='action', dest='action')
     onp = subp.add_parser('on', aliases=['start', 'add'],
@@ -201,6 +198,3 @@ def tracermain(args=None):
     ns = parser.parse_args(args)
     return _TRACER_MAIN_ACTIONS[ns.action](ns, args)
 
-
-if __name__ == '__main__':
-    tracermain()
